

'''
	import carbonado
	carbonado = carbonado.start ()
	
	carbonado.server.stop ()
'''

import glob
import inspect
import json
import os
import pathlib
import rich

import carbonado.basin as basin
from carbonado._clique import clique

import carbonado.climate as climate

	
import law_dictionary

def retrieve_directory ():
	return os.getcwd ()
	
	return os.path.dirname (
		os.path.abspath ((inspect.stack () [1]) [1])
	)


import carbonado.modules.add_glob_to_paths as add_glob_to_paths
		
	
def start (
	params = {},
	verbose = 1
):	
	the_climate = climate.scan ();
	preconfigured_extensions = the_climate ["preconfigured extensions"]

	if (verbose >= 1):
		print ()
		print ('"carbonado" is starting.')
		print ('module path is:', pathlib.Path (__file__).parent.resolve ())
		print ()

	rich.print_json (data = {
		"params before parsing": params
	})

	law_dictionary.check (	
		allow_extra_fields = True,
		laws = {
			"directory": {
				"required": False,
				"contingency": retrieve_directory,
				"type": str
			},
			"relative path": {
				"required": False,
				"contingency": retrieve_directory,
				"type": str
			},
			"port": {
				"required": False,
				"contingency": 2345,
				"type": int
			},
			"static port": {
				"required": False,
				"contingency": False,
				"type": bool 
			},
			"verbose": {
				"required": False,
				"contingency": 1,
				"type": int
			},
			"extensions": {
				"required": False,
				"contingency": preconfigured_extensions,
				"type": list
			}
		},
		dictionary = params
	)
	
	extensions = params ["extensions"]
	directory = params ["directory"]
	relative_path = params ["relative path"]
	static_port = params ["static port"]
	start_at_port = params ["port"]

	name_of_label = os.path.basename (directory)

	rich.print_json (data = {
		"params after parsing": params
	})


	paths = []
	for extension in extensions:
		add_glob_to_paths.start (
			directory = directory,
			extension = extension,
			relative_path = relative_path,
			verbose = verbose,
			
			paths = paths
		)
	
	
	if (verbose >= 1):
		print ("paths:", json.dumps (paths, indent = 4))
	
	the_server = basin.start (
		paths = paths,
		name_of_label = name_of_label,
		
		start_at_port = start_at_port,
		static_port = static_port
	)
	
	server = the_server ["server"]
	actual_port = the_server ["port"]
	
	server.start ()
	
	def stop ():
		if (verbose >= 1):
			print ('"carbonado" is stopping.')
	
		
		#server.stop ()
		termination = server.terminate ()
		#server.join ()
	
		if (verbose >= 1):
			print ('"carbonado" has stopped.')
			print ()
	
	import atexit
	atexit.register (stop)
	
	print ('The carbonado server has started')

	class Proceeds ():
		def __init__ (this, server, stop, port):
			this.stop = stop;
			this.port = port;
			

	return Proceeds (
		server,
		stop,
		actual_port
	)




import os
import time
import json
# os.getcwd()

import carbonado
import carbonado.climate as climate
	


def clique ():

	print ('starting clique')

	import click
	@click.group (invoke_without_command = True)
	@click.pass_context
	def group (ctx):
		if ctx.invoked_subcommand is None:
			#click.echo ('Clique was invoked without a subcommand.')
			start ([], standalone_mode = False)
		else:
			#click.echo ('Clique was invoked with the subcommand: %s' % ctx.invoked_subcommand)
			pass;
	
		pass

	the_climate = climate.scan ();
	preconfigured_extensions = the_climate ["preconfigured extensions"]

	# https://stackoverflow.com/questions/47631914/how-to-pass-several-list-of-arguments-to-click-option
	def parse_extensions (option):	
		try:
			option = json.loads (option)
			# option = str(option)  # this also works
		except ValueError:
			print ("ValueError:", ValueError)
			pass

		option = option [1:-1]  # trim '[' and ']'
		options = option.split (',')

		print ("options:", options)

		for i, value in enumerate (options):
			try:
				int(value)
			except ValueError:
				options[i] = value
			else:
				options[i] = int(value)
			
		trimmed_options = [string.strip() for string in options]
			
		return trimmed_options;

	'''
		carbonado carbonado --extensions [.s.HTML,.jpg,.png]
		carbonado carbonado --extensions "[ .s.HTML, .jpg, .png ]"
	'''
	import click
	@click.command ("carbonado")
	@click.option ('--extensions', default = preconfigured_extensions)
	@click.option ('--port', default = 6789)
	@click.option ('--static-port', is_flag = True, default = False)
	@click.option ('--verbose', default = 1)
	def internal_start (extensions, port, static_port, verbose):
		print ("carbonado")
		print ("extensions:", type (extensions), extensions)
		
		parsed_extensions = parse_extensions (extensions);
		
		
	
		print ("parsed exetnsions:", parsed_extensions)
	
		import pathlib
		from os.path import dirname, join, normpath
		this_dir = str (pathlib.Path (__file__).parent.resolve ())
	
		carbonado.start ({
			"directory": this_dir,
			"relative path": this_dir,
			
			"port": port,
			"static port": static_port,
			"verbose": verbose,
			
			"extensions": parsed_extensions
		})
		
		# close = input ("press close to exit") 
		while True:                                  
			time.sleep (1)  

	'''
		carbonado start --port 2345 --static-port
	'''
	import click
	@click.command ("start")
	@click.option ('--extensions', default = preconfigured_extensions)
	@click.option ('--port', default = 2345)
	@click.option ('--static-port', is_flag = True, default = False)
	def start (extensions, port, static_port):	
		parsed_extensions = parse_extensions (extensions)
	
		carbonado.start ({
			"directory": os.getcwd (),
			"relative path": os.getcwd (),
			
			"port": port,
			"static port": static_port,
			"verbose": 1,
			
			"extensions": parsed_extensions
		})
		
		# close = input ("press close to exit") 
		while True:                                  
			time.sleep (1)  

	group.add_command (internal_start)
	group.add_command (start)

	#start ([], standalone_mode=False)

	#group.add_command (clique_group ())
	group ()


	#start ()
	                          




import glob
import os


def start (
	directory = None,
	extension = None,
	relative_path = None,
	verbose = None,
	
	paths = None
):
	glob_param = directory + "/**/*" + extension;
	if (verbose >= 2):
		print ("glob:", glob_param)

	finds = glob.glob (glob_param, recursive = True)
	if (verbose >= 2):
		print ("finds:", json.dumps (finds, indent = 4))
	
	for find in finds:
		path = os.path.relpath (find, relative_path)
		name = path.split (extension) [0]
	
		paths.append ({
			"path": path,
			"name": name,
			"find": find
		})